@@ -40,6 +40,7 @@ from .function import (
     GIBBON,
     HIPPO,
     AugmentedExpectedImprovement,
+    BatchExpectedImprovement,
     BatchMonteCarloExpectedHypervolumeImprovement,
     BatchMonteCarloExpectedImprovement,
     BayesianActiveLearningByDisagreement,
@@ -49,6 +50,7 @@ from .function import (
     ExpectedHypervolumeImprovement,
     ExpectedImprovement,
     Fantasizer,
+    FastConstraintsFeasibility,
     GreedyContinuousThompsonSampling,
     IntegratedVarianceReduction,
     LocalPenalization,
@@ -64,10 +66,12 @@ from .function import (
     ProbabilityOfFeasibility,
     augmented_expected_improvement,
     batch_ehvi,
+    batch_expected_improvement,
     bayesian_active_learning_by_disagreement,
     bichon_ranjan_criterion,
     expected_hv_improvement,
     expected_improvement,
+    fast_constraints_feasibility,
     gibbon_quality_term,
     gibbon_repulsion_term,
     hard_local_penalizer,
@@ -76,7 +80,7 @@ from .function import (
     min_value_entropy_search,
     multiple_optimism_lower_confidence_bound,
     predictive_variance,
-    probability_of_feasibility,
+    probability_below_threshold,
     soft_local_penalizer,
 )
 from .interface import (
