@@ -19,7 +19,7 @@ This module contains functionality for optimizing
 
 from __future__ import annotations
 
-from typing import Any, Callable, List, Optional, Tuple, Union, cast
+from typing import Any, Callable, List, Optional, Sequence, Tuple, Union, cast
 
 import greenlet as gr
 import numpy as np
@@ -28,7 +28,14 @@ import tensorflow as tf
 import tensorflow_probability as tfp
 
 from .. import logging
-from ..space import Box, DiscreteSearchSpace, SearchSpace, SearchSpaceType, TaggedProductSearchSpace
+from ..space import (
+    Box,
+    Constraint,
+    DiscreteSearchSpace,
+    SearchSpace,
+    SearchSpaceType,
+    TaggedProductSearchSpace,
+)
 from ..types import TensorType
 from .interface import AcquisitionFunction
 
@@ -441,7 +448,9 @@ def _perform_parallel_continuous_optimization(
     # Set up child greenlets
     child_greenlets = [ScipyOptimizerGreenlet() for _ in range(num_optimization_runs)]
     vectorized_child_results: List[Union[spo.OptimizeResult, "np.ndarray[Any, Any]"]] = [
-        gr.switch(vectorized_starting_points[i].numpy(), bounds[i], optimizer_args)
+        gr.switch(
+            vectorized_starting_points[i].numpy(), bounds[i], space.constraints, optimizer_args
+        )
         for i, gr in enumerate(child_greenlets)
     ]
 
@@ -482,6 +491,11 @@ def _perform_parallel_continuous_optimization(
         [result.nfev for result in final_vectorized_child_results], dtype=tf_dtype
     )
 
+    # Ensure chosen points satisfy any constraints in the search-space.
+    if space.has_constraints:
+        is_feasible = space.is_feasible(vectorized_chosen_x)
+        vectorized_successes = tf.logical_and(vectorized_successes, is_feasible)
+
     successes = tf.reshape(vectorized_successes, [-1, V])  # [num_optimization_runs, V]
     fun_values = tf.reshape(vectorized_fun_values, [-1, V])  # [num_optimization_runs, V]
     chosen_x = tf.reshape(vectorized_chosen_x, [-1, V, D])  # [num_optimization_runs, V, D]
@@ -502,6 +516,7 @@ class ScipyOptimizerGreenlet(gr.greenlet):  # type: ignore[misc]
         self,
         start: "np.ndarray[Any, Any]",
         bounds: spo.Bounds,
+        constraints: Sequence[Constraint],
         optimizer_args: Optional[dict[str, Any]] = None,
     ) -> spo.OptimizeResult:
         cache_x = start + 1  # Any value different from `start`.
@@ -523,7 +538,10 @@ class ScipyOptimizerGreenlet(gr.greenlet):  # type: ignore[misc]
 
             return cast("np.ndarray[Any, Any]", cache_y), cast("np.ndarray[Any, Any]", cache_dy_dx)
 
-        optimizer_args = dict(dict(method="l-bfgs-b"), **(optimizer_args or {}))
+        method = "trust-constr" if len(constraints) else "l-bfgs-b"
+        optimizer_args = dict(
+            dict(method=method, constraints=constraints), **(optimizer_args or {})
+        )
         return spo.minimize(
             lambda x: value_and_gradient(x)[0],
             start,
